@@ -115,6 +115,13 @@ class Config:
             lockfile.rm(absent="ignore")
             raise e
 
+    def __getstate__(self):
+        return self.__dict__
+
+    def __setstate__(self, d: dict):
+        for item in d:
+            setattr(self, item, d[item])
+
     def __getattr__(self, item):
         if item == "configs":
             raise ValueError("Config object's 'configs' not set")
@@ -200,11 +207,6 @@ class NodeLearningConfig(Config):
         key *x_maxs*, but with value "".
     max_depth: int
         Maximum depth of the decision tree to use in the model
-    remember_activation: bool
-        If True, the model created by the node will remember its activation (logical OR of all its rules)
-    stack_activation: bool
-        If True, the model created by the node will stack its rules' activation vectors into a np.ndarray (watch out
-        for memory overload)
     plot_data: bool
         If True, nodes will plot distributions of their data in the same directory they found the said data.
         Plots will beproduced at each learning triggered by a central model update, and will be done both for datapreped
@@ -267,8 +269,6 @@ class NodeLearningConfig(Config):
         "x_mins",
         "x_maxs",
         "max_depth",
-        "remember_activation",
-        "stack_activation",
         "plot_data",
         "get_leaf",
         "node_models_path",
@@ -349,6 +349,15 @@ class AggregatorConfig(Config):
     emitter_path_fs: TransparentPath
         File system of path to emitter. Can be 'gcs', 'local' or "". If not specified, the json
         file should still contain the key *emitter_path_fs*, but with value "".
+    weight: str
+        If using classification rules, 'weight' is unused. If using regression rules, will be used to average duplicated
+        rules' prediction when aggregating. Can be 'equi' or any rlue attribute name that can be used as a weight
+        (coverage, criterion...). If not specified, the json file should still contain the key *weight*,
+        but with value "". In which case rules will be equally weighted.
+    best: str
+        If using classification rules, 'best' is unused. If using regression rules, specify whether the rule is best
+        when its 'weight' is big ('best'="max") or small ('best'="min").If not specified, the json file should still
+        contain the key *best*, but with value "". In which case, it will be set to "max".
     """
 
     EXPECTED_CONFIGS = [
@@ -361,8 +370,19 @@ class AggregatorConfig(Config):
         "aggregation_kwargs",
         "emitter_path",
         "emitter_path_fs",
+        "weight",
+        "best"
     ]
 
+    def __init__(self, path: Optional[Union[str, Path, TransparentPath]] = None):
+        super().__init__(path)
+        if self.weight is None:
+            self.weight = "equi"
+        if self.best is None:
+            self.best = "max"
+        if self.best != "min" and self.best != "max":
+            raise ValueError(f"Attribute 'best' can be 'min' or 'max', not '{self.best}'")
+
 
 class CentralConfig(Config):
     # noinspection PyUnresolvedReferences
