@@ -12,7 +12,8 @@ import logging
 
 from .actor import Actor
 from .configs import NodeLearningConfig, NodeDataConfig
-from .fitters import DecisionTreeFitter, Fitter
+from .diff_privacy import apply_diff_privacy
+from .fitters import DecisionTreeClassificationFitter, Fitter, DecisionTreeRegressionFitter
 from .node_model_updaters import AdaBoostNodeModelUpdater, NodeModelUpdater
 from .datapreps import BinFeaturesDataPrep, DataPrep
 from .train_test_split import TrainTestSplit
@@ -65,7 +66,10 @@ class Node(Actor):
         Configuration of the paths to the node's features and target data files
     """
 
-    possible_fitters = {"decisiontree": DecisionTreeFitter}
+    possible_fitters = {
+        "decisiontreeclassification": DecisionTreeClassificationFitter,
+        "decisiontreeregression": DecisionTreeRegressionFitter
+    }
     """Possible string values and corresponding fitter object for *fitter* attribute of
     `ifra.configs.NodeLearningConfig`"""
 
@@ -223,44 +227,53 @@ class Node(Actor):
         """
 
         self.plot_dataprep_and_split()
+
+        x = self.data.x_train_path.read(**self.data.x_read_kwargs).values
+        y = self.data.y_train_path.read(**self.data.y_read_kwargs).squeeze().values
+        if self.data.x_test_path != self.data.x_train_path:
+            x_test = self.data.x_test_path.read(**self.data.x_read_kwargs).values
+        else:
+            x_test = None
+        if self.data.y_test_path != self.data.y_train_path:
+            y_test = self.data.y_test_path.read(**self.data.y_read_kwargs).squeeze().values
+        else:
+            y_test = y
+
         logger.info(f"Fitting node {self.learning_configs.id} using {self.learning_configs.fitter}...")
-        model = self.fitter.fit(
-            self.data.x_train_path.read(**self.data.x_read_kwargs).values,
-            self.data.y_train_path.read(**self.data.y_read_kwargs).values,
-        )
-        good_rules = [r for r in model if r.good]
+        if len(x) == 0:
+            logger.warning(f"No data in node {self.learning_configs.id}'s features")
+            return
+        self.model = self.fitter.fit(x=x, y=y)
+        if self.model is None:
+            logger.warning(f"Fitter could not produce a model in node {self.learning_configs.id}")
+            return
         central_model_path = self.learning_configs.central_model_path
         if central_model_path.isfile():
             central_model = RuleSet()
             central_model.load(central_model_path)
-            rules = [r for r in good_rules if r not in central_model]
-            if len(rules) > 0:
-                self.model = RuleSet(
-                    remember_activation=model.remember_activation,
-                    stack_activation=model.stack_activation,
-                    rules_list=rules
-                )
-                logger.info(f"Found {len(self.model)} new good rules in node {self.learning_configs.id}")
-                self.model_to_file()
-                self.fitter.save(
-                    self.learning_configs.node_models_path / f"model_{self.filenumber}_{self.iteration}"
-                )
-            else:
-                logger.info(f"Did not find new rules in node {self.learning_configs.id}")
+            rules = [r for r in self.model if r not in central_model]
+            self.model = RuleSet(rules)
+
+        if len(self.model) > 0:
+            self.coverage = self.model.ruleset_coverage
+            self.model.eval(
+                xs=x_test,
+                y=y_test,
+                keep_new_activations=x_test is not None
+            )
+            # noinspection PyProtectedMember
+            logger.info(
+                f"Found {len(self.model)} new good rules in node {self.learning_configs.id}."
+                f" Criterion is {round(self.model.criterion, 3)}, coverage is {round(self.coverage, 3)}"
+            )
+            self.model_to_file()
+            self.fitter.save(
+                self.learning_configs.node_models_path / f"model_{self.filenumber}_{self.iteration}"
+            )
         else:
-            self.model = RuleSet(
-                    remember_activation=model.remember_activation,
-                    stack_activation=model.stack_activation,
-                    rules_list=good_rules
-                )
-            if len(self.model) == 0:
-                logger.info(f"No good rules found in node {self.learning_configs.id}")
-            else:
-                logger.info(f"Found {len(self.model)} rules in node {self.learning_configs.id}")
-                self.model_to_file()
-                self.fitter.save(
-                    self.learning_configs.node_models_path / f"model_{self.filenumber}_{self.iteration}"
-                )
+            logger.info(f"Did not find rules in node {self.learning_configs.id}")
+        if self.model is not None:
+            apply_diff_privacy(ruleset=self.model, y=y)
 
     @emit
     def update_from_central(self, model: RuleSet) -> None:
@@ -291,6 +304,8 @@ class Node(Actor):
         if model is None:
             return
 
+        if not self.learning_configs.node_models_path.isdir():
+            self.learning_configs.node_models_path.mkdir(parents=True)
         existing_files = list(self.learning_configs.node_models_path.glob('model_main_*.csv'))
         if self.filenumber is None:
             self.filenumber = random.randint(0, int(1e6))
@@ -305,6 +320,8 @@ class Node(Actor):
         path_iteration = self.learning_configs.node_models_path / f"model_{self.filenumber}_{self.iteration}.csv"
         self.iteration += 1
 
+        if not path_main.parent.isdir():
+            path_main.parent.mkdir(parents=True)
         model.save(path_main)
         model.save(path_iteration)
 
@@ -345,7 +362,7 @@ class Node(Actor):
                 path_x = path_x.parent / f"{name}_{iteration}.pdf"
             fig.savefig(path_x)
 
-        y = self.data.y_path.read(**self.data.y_read_kwargs)
+        y = self.data.y_path.read(**self.data.y_read_kwargs).squeeze()
         fig = plot_histogram(data=y.squeeze(), xlabel="Class", figsize=(10, 7))
 
         iteration = 0
@@ -383,8 +400,9 @@ class Node(Actor):
             self.update_from_central(central_model)
             logger.info(f"Fetched central model in node {self.learning_configs.id} at {self.last_fetch}")
 
-        t = time()
         do_fit = True  # start at true to trigger fit even if no central model is here at first iteration
+        self.plot_dataprep_and_split()  # make dataprep at run start
+        started = False  # To force at least one loop of the while to trigger
 
         if timeout <= 0:
             logger.warning("You did not specify a timeout for your run. It will last until manually stopped.")
@@ -395,8 +413,9 @@ class Node(Actor):
             f"{self.learning_configs.central_model_path}, {self.data.x_path} and {self.data.y_path}."
         )
 
-        while time() - t < timeout or timeout <= 0:
-
+        t = time()
+        while time() - t < timeout or timeout <= 0 or started is False:
+            started = True
             if (
                 self.last_x is not None
                 and self.last_y is not None
